
WIDTH = 640 -- screen dimensions
HEIGHT = 960
MIDX = 320
MIDY = 480

MIN_ANGLE = 360 - 60 -- stone path constraints
ARC_WIDTH = 60 * 2

NUMSTONES = 50 -- length of generated stone path

BUMP = require('cbump').newWorld(256)

local score = 0

local camera = Sprite.new()
camera.position = {x = 0, y = 0}

local climberPack = TexturePack.new('climber.txt', 'climber.png')
local climber

-- CLIMBER (PLAYER) CLASS

Climber = Core.class(Bitmap)

function Climber:init(texture)
  -- calculate anchorpoints between 0 and 1
  local w = 1 / self:getWidth()
  local h = 1 / self:getHeight()
  self.anchors = {
    left = {x = w * 157, y = h * 36},
    right = {x = w * 157, y = h * 292},
  }
  self.distanceBetweenHands = 256
  self:reset()
end

function Climber:reset()
  local anchors = self.anchors
  local anchor = anchors.right
  self:setAnchorPoint(anchor.x, anchor.y)

  self:setRotation(0)
  self.rotSpeed = 1.5
  self.clockwise = true

  local pack = climberPack
  self:setTextureRegion(climberPack:getTextureRegion('climber_cw.png'))

  local x, y = MIDX, MIDY
  self.position = {x = x, y = y}
  self.hand = {x = 0, y = 0}
  self:setPosition(x, y)
end

function Climber:changeDirection(stone)
  local cos, sin = math.cos, math.sin
  local pack = climberPack

  local clockwise = not self.clockwise

  local a = self:getRotation()
  local d = self.distanceBetweenHands

  local anchors = self.anchors
  local anchor = anchors.left
  local texture
  if clockwise then
    a -= 180
    anchor = anchors.right
    texture = pack:getTextureRegion('climber_cw.png')
  else
    texture = pack:getTextureRegion('climber_ccw.png')
  end

  self:setAnchorPoint(anchor.x, anchor.y)
  self:setTextureRegion(texture)

  local hx = cos(^<(a - 90)) * d
  local hy = sin(^<(a - 90)) * d

  local cx, cy = self:getPosition()

  cx,cy = stone:getPosition()

  self:setPosition(cx, cy)--cx + hx, cy + hy)
  
  self.clockwise = clockwise
end

function Climber:update(dt) -- update every frame
  local cos, sin = math.cos, math.sin

  -- rotate player
  local r = self:getRotation()
  local rotSpeed = self.rotSpeed
  if self.clockwise then
    r += rotSpeed
  else
    r -= rotSpeed
  end
  self:setRotation(r % 360)

  -- calculate grabbing hand position
  local hand = self.hand

  local cx, cy = climber:getPosition()
  local a = climber:getRotation()
  if not climber.clockwise then
    a -= 180
  end
  local d = climber.distanceBetweenHands

  local hx = cos(^<(a - 90)) * d
  local hy = sin(^<(a - 90)) * d
  local x, y = hx + cx, hy + cy

  hand.x, hand.y = x, y
end

climber = Climber.new(climberPack:getTextureRegion('climber_cw.png'))

-- STONES

local stones = {}

local function genStones()
  local cos, sin, random = math.cos, math.sin, math.random
  local texture = Texture.new('stone.png', true)

  local bump = BUMP
  local d = climber.distanceBetweenHands
  local cx, cy = climber:getPosition()
  for i = 1, NUMSTONES do
    local a = (MIN_ANGLE + (random() * ARC_WIDTH)) % 360
    local hx = cos(^<(a - 90)) * d
    local hy = sin(^<(a - 90)) * d

    if #stones == 0 then
      cx, cy = MIDX, MIDY -- the first stone is always placed here
      hx, hy = 0, 0
    else
      cx, cy = stones[#stones]:getPosition() -- other stones placed relative to previous stones
    end

    local stone = Bitmap.new(texture)
    stone:setAnchorPoint(0.5, 0.5)
    local x, y = cx + hx, cy + hy
    stone:setPosition(x, y)
    camera:addChild(stone)

    stones[#stones + 1]= stone
--    print(x, y)

    -- add bump collision rect and visual indicator (a Pixel)
    local w, h = stone:getWidth(), stone:getHeight()
    bump:add(stone, x - (w * 0.25), y - (h * 0.25), w * 0.5, h * 0.5)    
    local p = Pixel.new(0x4488ff, 0.5, w * 0.5, h * 0.5)
    p:setAnchorPoint(0.5, 0.5)
    stone:addChild(p)

  end

end

local function resetStones()
  local bump = BUMP
  for i = 1, #stones do
    local stone = stones[i]
    stone:getChildAt(1):removeFromParent() -- remove Pixel
    stone:removeFromParent()
    bump:remove(stone)
  end
  stones = {} -- empty list
end

genStones()

camera:addChild(climber)

-- CREATE A SCROLLING BACKGROUND
local random = math.random
local backTexture = Texture.new('background.png', true, {wrap = Texture.REPEAT})
local region = TextureRegion.new(backTexture, 0, 0, WIDTH, HEIGHT)
local back = Bitmap.new(region)
back.region = region
back:setColorTransform(random(), random(), random())
stage:addChild(back)

local scoreText = TextField.new(nil, '0')
scoreText:setScale(16)
scoreText:setTextColor(0xe0e7ef)
scoreText:setPosition(16, 128)

-- RESET THE ENTIRE GAME
local function resetGame()
  local random = math.random
  climber:reset()
  camera:setPosition(0, 0)
  resetStones()
  genStones()
  back:setColorTransform(random(), random(), random())
  --climber:removeFromParent() -- bring climber back to front
  camera:addChild(climber)

  score = 0
  scoreText:setText('0')  
end



-- UPDATE EVERY FRAME
local function onEnterFrameFunc(e)
  local cos, sin = math.cos, math.sin

  local dt = e.deltaTime
  climber:update(dt)

  -- keep climber in view
  local cx, cy = climber:getPosition()
  local a = climber:getRotation()
  if not climber.clockwise then
    a -= 180
  end
  local d = climber.distanceBetweenHands * 1.25

  local hx = cos(^<(a - 90)) * d
  local hy = sin(^<(a - 90)) * d
  local x, y = hx + cx, hy + cy
  
  local cp = camera.position
  cx, cy = cp.x, cp.y

  if x < cx then
    cx = x
  elseif x > cx + WIDTH then
    cx = x - WIDTH
  end

  if y < cy then
    cy = y
  elseif y > cy + HEIGHT then
    cy = y - HEIGHT
  end

  cp.x, cp.y = cx, cy
  camera:setPosition(-cx, -cy)
  
  -- scroll background
  local region = back.region
  region:setRegion(cx % 512, cy % 512, WIDTH, HEIGHT)
  back:setTextureRegion(region)
end
stage:addEventListener(Event.ENTER_FRAME, onEnterFrameFunc)

-- PROCESS TOUCH EVENT
local function onTapFunc(event)
  -- determine of climber hand over rock.. if so then grab
  -- rock and change direction.. otherwise reset the game.
  local bump = BUMP
  local hand = climber.hand
  local x, y = hand.x, hand.y
  local items, len = bump:queryPoint(x, y)
  if #items > 0 then
    climber:changeDirection(items[1]) -- climber advanced!
    score += 1
    scoreText:setText(score)
  else
    resetGame() -- climber fell to their death :(
  end
end

stage:addEventListener(Event.MOUSE_DOWN, onTapFunc,self)

stage:addChild(camera)

stage:addChild(scoreText)
